@@ -5,15 +5,19 @@
 ;;;                       Copyright (c) 1996,1997                         ;;
 ;;;                        All Rights Reserved.                           ;;
 ;;;                                                                       ;;
-;;;  Permission to use, copy, modify, distribute this software and its    ;;
-;;;  documentation for research, educational and individual use only, is  ;;
-;;;  hereby granted without fee, subject to the following conditions:     ;;
+;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
+;;;  this software and its documentation without restriction, including   ;;
+;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
+;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
+;;;  permit persons to whom this work is furnished to do so, subject to   ;;
+;;;  the following conditions:                                            ;;
 ;;;   1. The code must retain the above copyright notice, this list of    ;;
 ;;;      conditions and the following disclaimer.                         ;;
 ;;;   2. Any modifications must be clearly marked as such.                ;;
 ;;;   3. Original authors' names are not deleted.                         ;;
-;;;  This software may not be used for commercial purposes without        ;;
-;;;  specific prior written permission from the authors.                  ;;
+;;;   4. The authors' names are not used to endorse or promote products   ;;
+;;;      derived from this software without specific prior written        ;;
+;;;      permission.                                                      ;;
 ;;;                                                                       ;;
 ;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
 ;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
@@ -29,30 +33,6 @@
 ;;;  General Festival Scheme specific functions
 ;;;  Including definitions of various standard variables.
 
-(defvar hush_startup nil
-  "hush_startup
-  If set to non-nil, the copyright banner is not displayed at start up.")
-
-(defvar readline_histsize 256
-  "readline_histsize
-  The number of line so be saved in the users history file when a 
-  Festival session ends.  The histfile is \".festival_history\" in the
-  users home directory.  Note this value is only checked when the 
-  command interpreter is started, hence this should be set in a user's
-  \".festivalrc\" or system init file.  Reseting it at the command
-  interpreter will have no effect.")
-
-(defvar ! nil
-  "!
-  In interactive mode, this variable's value is the return value of the
-  previously evaluated expression.")
-
-(defvar Parameter nil
-  "Parameter
-  An assoc-list of parameters and values for various parts of the speech
-  synthesis system.  This is used by the functions Parameter.set 
-  Parameter.def and Parameter.get as well as internal C++ functions.")
-
 ;; will be set automatically on start-up
 (defvar festival_version "unknown"
   "festival_version
@@ -64,33 +44,6 @@
  A list of major, minor and subminor version numbers of the current
  system.  e.g. (1 0 12).")
 
-(define (Parameter.set name val)
-"(Parameter.set NAME VAL)
-  Set parameter NAME to VAL (deleting any previous setting)"
-  (let ((pair (assoc name Parameter)))
-   (if pair
-       (set-car! (cdr pair) val)
-       (set! Parameter (cons (list name val) Parameter)))
-   val))
-
-(define (Parameter.def name val)
-"(Parameter.def NAME VAL)
-  Set parameter NAME to VAL if not already set"
-  (let ((pair (assoc name Parameter)))
-   (if pair
-       (car (cdr pair))
-       (and
-	(set! Parameter (cons (list name val) Parameter))
-	val))))
-
-(define (Parameter.get name)
-"(Parameter.get NAME)
-  Get parameter NAME's value (nil if unset)"
-  (let ((pair (assoc name Parameter)))
-    (if pair
-	(car (cdr pair))
-	nil)))
-
 (define (apply_method method utt)
 "(apply_method METHOD UTT)
 Apply the appropriate function to utt defined in parameter."
@@ -100,51 +53,36 @@ Apply the appropriate function to utt defined in parameter."
       nil)   ;; should be an error, but I'll let you off at present
      ((and (symbol? method_val) (symbol-bound? method_val))
       (apply (symbol-value method_val) (list utt)))
-     ((not (symbol? method_val))
+     ((member (typeof method_val) '(subr closure))
       (apply method_val (list utt)))
      (t      ;; again is probably an error
       nil))))
 
-(defvar ufoutfp nil)
+(define (require_module l)
+  "(require_module l)
+Check that certain compile-time modules are included in this installation.
+l may be a single atom or list of atoms.  Each item in l must appear in
+*modules* otherwise an error is throw."
+  (if (consp l)
+      (mapcar require_module l)
+      (if (not (member_string l *modules*))
+	  (error (format nil "module %s required, but not compiled in this installation\n" l))))
+  t)
 
 ;;;  Feature Function Functions
-(define (utt.features utt stream func_list)
-"(utt.features UTT STREAMNAME FUNCLIST)
-  Get vectors of feature values for each item in STREAMNAME in UTT.
-  If ufoutfp is non-nil write them to that file else return them as a list.
+(define (utt.features utt relname func_list)
+"(utt.features UTT RELATIONNAME FUNCLIST)
+  Get vectors of feature values for each item in RELATIONNAME in UTT.
   [see Features]"
-  (if ufoutfp
-      (and 
-       (mapcar
-	(lambda (s)
-	  (mapcar 
-	   (lambda (f)
-	     (format ufoutfp "%l " (utt.streamitem.feat utt s f)))
-	   func_list)
-	  (terpri ufoutfp))
-	(utt.stream utt stream)) t)
-      (mapcar (lambda (s) (mapcar (lambda (f)
-				    (utt.streamitem.feat utt s f))
-				  func_list))
-	      (utt.stream utt stream))))
-
-(define (dump_feats stream features names outfile)
-"(dump_feats STREAMNAME FEATURES NAMES OUTFILE)
-  Dump FEATURES of STREAMNAME in each utterances in NAMES to OUTFILE.
-  Utterances are created by calling test_seg on each member of NAMES."
-  (set! ufoutfp (fopen outfile "w"))
-  (mapcar
-   (lambda (name)
-     (print name)
-     (utt.features (test_seg name) stream features))
-   names)
-  (fclose ufoutfp)
-  (set! ufoutfp nil))
+  (mapcar 
+   (lambda (s) 
+     (mapcar (lambda (f) (item.feat s f)) func_list))
+   (utt.relation.items utt relname)))
 
 (define (utt.type utt)
 "(utt.type UTT)
   Returns the type of UTT."
-  (intern (utt.streamitem.feat utt (car (utt.stream utt 'IForm)) 'type)))
+  (intern (utt.feat utt 'type)))
 
 (define (utt.save.segs utt filename)
 "(utt.save.segs UTT FILE)
@@ -154,33 +92,19 @@ Apply the appropriate function to utt defined in parameter."
     (mapcar
      (lambda (info)
        (format fd "%2.4f 100 %s\n" (car info) (car (cdr info))))
-     (utt.features utt 'Segment '(end name)))
+     (utt.features utt 'Segment '(segment_end name)))
     (fclose fd)
     utt))
 
-(define (utt.save.stream utt streamname filename)
-"(utt.save.stream UTT STREAMNAME FILE)
-  Save items in STREAMNAME of UTT in a FILE in xlabel format."
+(define (utt.save.words utt filename)
+"(utt.save.words UTT FILE)
+  Save words of UTT in a FILE in xlabel format."
   (let ((fd (fopen filename "w")))
     (format fd "#\n")
     (mapcar
      (lambda (info)
        (format fd "%2.4f 100 %s\n" (car info) (car (cdr info))))
-     (utt.features utt streamname '(end name)))
-    (fclose fd)
-    utt))
-
-(define (utt.save.units utt filename)
-"(utt.save.units UTT FILE)
-  Save selected unit descriptions of UTT in a FILE in xlabel format."
-  (let ((fd (fopen filename "w")))
-    (format fd "#\n")
-    (mapcar
-     (lambda (info)
-       (format fd "%2.4f 100 %l\n" 
-	       (car info) 
-	       (ducs.unit (car (cdr info)))))
-     (utt.features utt 'Segment '(end unit_id)))
+     (utt.features utt 'Word '(word_end name)))
     (fclose fd)
     utt))
 
@@ -191,24 +115,268 @@ supported by the Speech Tool Library).   This loads, synthesizes and
 plays the utterance."
    (utt.play (utt.synth (utt.load.segf0 labfile f0file))))
 
-(define (utt.stream.present utt streamname)
-"(utt.stream.present UTT STREAMNAME)
-Returns t if UTT caontaisn a stream call STREAMNAME, nil otherwise."
-  (if (member name (utt.streamnames utt))
+(define (utt.relation.present utt relation)
+"(utt.relation.present UTT RELATIONNAME)
+Returns t if UTT caontains a relation called RELATIONNAME, nil otherwise."
+  (if (member_string relation (utt.relationnames utt))
       t
       nil))
 
-(define (stream.set_duration streamitem duration)
-"(stream.set_duration STREAMITEM DURATION)
-Sets STREAMITEMS duration to DURATION.  Note care should be take with
-this as duration is derived from the end feature.  It is the user's
-responsibility to ensure that durations do not cause end times through a
-stream to cease to be unordered."
-  (streamitem.set_end
-   streamitem
-   (+ duration
-      (if (streamitem.prev streamitem)
-	  (streamitem.end streamitem)
+(define (utt.relation.leafs utt relation)
+"(utt.relation.leafs UTT RELATIONNAME)
+Returns a list of all the leafs in this relation."
+  (let ((leafs nil))
+    (mapcar
+     (lambda (i)
+       (if (not (item.down (item.relation i relation)))
+	   (set! leafs (cons i leafs))))
+     (utt.relation.items utt relation))
+    (reverse leafs)))
+
+(define (utt.relation.first utt relation)
+"(utt.relation.first UTT RELATIONNAME)
+Returns a the first item in this relation."
+  (utt.relation utt relation))
+
+(define (utt.relation.last utt relation)
+"(utt.relation.last UTT RELATIONNAME)
+Returns a the last item in this relation."
+  (let ((i (utt.relation.first utt relation)))
+    (while (item.next i)
+	   (set! i (item.next i)))
+    i))
+
+(define (item.relation.append_daughter parent relname daughter)
+"(item.relation.append_daughter parent relname daughter)
+Make add daughter to parent as a new daughter in relname."
+   (item.append_daughter (item.relation parent relname) daughter))
+
+(define (item.relation.insert si relname newsi direction)
+"(item.relation.insert si relname newsi direction)
+Insert newsi in relation relname with respect to direction.  If
+direction is ommited after is assumed, valid directions are after
+before, above and below.  Note you should use 
+item.relation.append_daughter for tree adjoining.  newsi maybe
+a item itself of a LISP description of one."
+   (item.insert 
+    (item.relation si relname)
+    newsi
+    direction))
+
+(define (item.relation.daughters parent relname)
+  "(item.relation.daughters parent relname)
+Return a list of all daughters of parent by relname."
+  (let ((d1 (item.daughter1 (item.relation parent relname)))
+	(daughters))
+    (while d1
+	   (set! daughters (cons d1 daughters))
+	   (set! d1 (item.next d1)))
+    (reverse daughters)))
+
+(define (item.daughters p)
+  "(item.daughters parent)
+Return a list of all daughters of parent."
+  (item.relation.daughters p (item.relation.name p)))
+
+(define (item.relation.parent si relname)
+  "(item.relation.parent item relname)
+Return the parent of this item in this relation."
+  (item.parent (item.relation si relname)))
+
+(define (item.relation.daughter1 si relname)
+  "(item.relation.daughter1 item relname)
+Return the first daughter of this item in this relation."
+  (item.daughter1 (item.relation si relname)))
+
+(define (item.relation.daughter2 si relname)
+  "(item.relation.daughter2 item relname)
+Return the second daughter of this item in this relation."
+  (item.daughter2 (item.relation si relname)))
+
+(define (item.relation.daughtern si relname)
+  "(item.relation.daughtern item relname)
+Return the final daughter of this item in this relation."
+  (item.daughtern (item.relation si relname)))
+
+(define (item.relation.next si relname)
+  "(item.relation.next item relname)
+Return the next item in this relation."
+  (item.next (item.relation si relname)))
+
+(define (item.relation.prev si relname)
+  "(item.relation.prev item relname)
+Return the previous item in this relation."
+  (item.prev (item.relation si relname)))
+
+(define (item.relation.first si relname)
+  "(item.relation.first item relname)
+Return the most previous item from this item in this relation."
+  (let ((n (item.relation si relname)))
+    (while (item.prev n)
+     (set! n (item.prev n)))
+    n))
+
+(define (item.leafs si)
+  "(item.relation.leafs item relname)
+Return a list of the leafs of this item in this relation."
+  (let ((ls nil)
+	(pl (item.first_leaf si))
+	(ll (item.next_leaf (item.last_leaf si))))
+    (while (and pl (not (equal? pl ll)))
+	   (set! ls (cons pl ls))
+	   (set! pl (item.next_leaf pl)))
+    (reverse ls)))
+
+(define (item.relation.leafs si relname)
+  "(item.relation.leafs item relname)
+Return a list of the leafs of this item in this relation."
+  (item.leafs (item.relation si relname)))
+
+(define (item.root s)
+  "(item.root s)
+Follow parent link until s has no parent."
+  (cond
+   ((item.parent s) 
+    (item.root (item.parent s)))
+   (t s)))
+
+(define (item.parent_to s relname)
+  "(item.parent_to s relname)
+Find the first ancestor of s in its current relation that is also in
+relname.  s is treated as an ancestor of itself so if s is in relname
+it is returned.  The returned value is in will be in relation relname
+or nil if there isn't one."
+  (cond
+   ((null s) s)
+   ((member_string relname (item.relations s)) 
+    (item.relation s relname))
+   (t (item.parent_to (item.parent s) relname))))
+
+(define (item.daughter1_to s relname)
+  "(item.daughter1_to s relname)
+Follow daughter1 links of s in its current relation until an item
+is found that is also in relname, is s is in relname it is returned.
+The return item is returned in relation relname, or nil if there is
+nothing in relname."
+  (cond
+   ((null s) s)
+   ((member_string relname (item.relations s)) (item.relation s relname))
+   (t (item.daughter1_to (item.daughter1 s) relname))))
+
+(define (item.daughtern_to s relname)
+  "(item.daughter1_to s relname)
+Follow daughtern links of s in its current relation until an item
+is found that is also in relname, is s is in relname it is returned.
+The return item is returned in relation relname, or nil if there is
+nothing in relname."
+  (cond
+   ((null s) s)
+   ((member_string relname (item.relations s)) (item.relation s relname))
+   (t (item.daughtern_to (item.daughtern s) relname))))
+
+(define (item.name s)
+"(item.name ITEM)
+  Returns the name of ITEM. [see Accessing an utterance]"
+  (item.feat s "name"))
+
+(define (utt.wave utt)
+  "(utt.wave UTT)
+Get waveform from wave (R:Wave.first.wave)."
+  (item.feat (utt.relation.first utt "Wave") "wave"))
+
+(define (utt.wave.rescale . args)
+ "(utt.wave.rescale UTT FACTOR NORMALIZE)
+Modify the gain of the waveform in UTT by GAIN.  If NORMALIZE is
+specified and non-nil the waveform is maximized first."
+  (wave.rescale (utt.wave (nth 0 args)) (nth 1 args) (nth 2 args))
+  (nth 0 args))
+
+(define (utt.wave.resample utt rate)
+  "(utt.wave.resample UTT RATE)\
+Resample waveform in UTT to RATE (if it is already at that rate it remains
+unchanged)."
+  (wave.resample (utt.wave utt) rate)
+  utt)
+
+(define (utt.import.wave . args)
+  "(utt.import.wave UTT FILENAME APPEND)
+Load waveform in FILENAME into UTT in R:Wave.first.wave.  If APPEND
+is specified and non-nil append this to the current waveform."
+  (let ((utt (nth 0 args))
+	(filename (nth 1 args))
+	(append (nth 2 args)))
+    (if (and append (member 'Wave (utt.relationnames utt)))
+	(wave.append (utt.wave utt) (wave.load filename))
+	(begin
+	  (utt.relation.create utt 'Wave)
+	  (item.set_feat
+	   (utt.relation.append utt 'Wave)
+	   "wave"
+	   (wave.load filename))))
+    utt))
+
+(define (utt.save.wave . args)
+  "(utt.save.wave UTT FILENAME FILETYPE)
+Save waveform in UTT in FILENAME with FILETYPE (if specified) or
+using global parameter Wavefiletype."
+  (wave.save 
+   (utt.wave (nth 0 args))
+   (nth 1 args)
+   (nth 2 args))
+  (nth 0 args))
+
+(define (utt.play utt)
+  "(utt.play UTT)
+Play waveform in utt by current audio method."
+  (wave.play (utt.wave utt))
+  utt)
+
+(define (utt.save.track utt filename relation feature)
+  "(utt.save.track utt filename relation feature)
+DEPRICATED use trace.save instead."
+  (format stderr "utt.save.track: DEPRICATED use track.save instead\n")
+  (track.save 
+   (item.feat
+    (utt.relation.first utt relation)
+    feature)
+   filename)
+  utt)
+
+(define (utt.import.track utt filename relation fname)
+  "(utt.import.track UTT FILENAME RELATION FEATURE_NAME)
+Load track in FILENAME into UTT in R:RELATION.first.FEATURE_NAME.
+Deletes RELATION if it already exists. (you maybe want to use track.load
+directly rather than this legacy function."
+  (utt.relation.create utt relation)
+  (item.set_feat
+   (utt.relation.append utt relation)
+   fname
+   (track.load filename))
+  utt)
+
+(define (wagon_predict item tree)
+"(wagon_predict ITEM TREE)
+Predict with given ITEM and CART tree and return the prediction
+(the last item) rather than whole probability distribution."
+ (car (last (wagon item tree))))
+
+(define (phone_is_silence phone)
+  (member_string 
+   phone
+   (car (cdr (car (PhoneSet.description '(silences)))))))
+
+(define (phone_feature phone feat)
+"(phone_feature phone feat)
+Return the feature for given phone in current phone set, or 0
+if it doesn't exist."
+  (let ((ph (intern phone)))
+    (let ((fnames (cadr (assoc 'features (PhoneSet.description))))
+	  (fvals (cdr (assoc ph (cadr (assoc 'phones (PhoneSet.description)))))))
+      (while (and fnames (not (string-equal feat (car (car fnames)))))
+	     (set! fvals (cdr fvals))
+	     (set! fnames (cdr fnames)))
+      (if fnames
+	  (car fvals)
 	  0))))
 
 (defvar server_max_clients 10
@@ -268,9 +436,9 @@ documentation string."
       (set! ff_docstrings (cons (cons fname fdoc) ff_docstrings))))
     t))
 
-(define (linear_regression utt item model)
-  "(linear_regression UTT ITEM MODEL)
-Use linear regression MODEL on UTT and ITEM.  MODEL consists of a list
+(define (linear_regression item model)
+  "(linear_regression ITEM MODEL)
+Use linear regression MODEL on ITEM.  MODEL consists of a list
 of features, weights and optional map list.  E.g. ((Intercept 100)
 (tobi_accent 10 (H* !H*)))."
   (let ((intercept (if (equal? 'Intercept (car (car model))) 
@@ -281,7 +449,7 @@ of features, weights and optional map list.  E.g. ((Intercept 100)
    (cons intercept
    (mapcar
     (lambda (f)
-     (let ((ff (utt.streamitem.feat utt item (car f))))
+     (let ((ff (item.feat item (car f))))
       (if (car (cdr (cdr f)))
          (if (member_string ff (car (cdr (cdr f))))
            (car (cdr f))
@@ -297,7 +465,7 @@ Getting Help
   (manual nil)      displays manual in local netscape
   C-c               return to top level
   C-d or (quit)     Exit Festival
-(If compiled with readline)
+(If compiled with editline)
   M-h               desplays help on current symbol  
   M-s               speaks help on current symbol  
   M-m               displays relevant manula page in local netscape
@@ -314,7 +482,7 @@ Doing stuff
   (tts FILENAME nil)  Say contexts of file, FILENAME should be 
                       surrounded by double quotes
   (voice_rab_diphone) Select voice (Britsh Male)
-  (voice_kd_diphone)  Select voice (American Male)
+  (voice_ked_diphone) Select voice (American Male)
 ")
 
 (define (festival_warranty)
@@ -327,18 +495,22 @@ Doing stuff
 "
                 Centre for Speech Technology Research                  
                      University of Edinburgh, UK                       
-                       Copyright (c) 1996,1997                         
+                       Copyright (c) 1996-1999                         
                         All Rights Reserved.                           
                                                                        
-  Permission to use, copy, modify, distribute this software and its            
-  documentation for research, educational and individual use only, is  
-  hereby granted without fee, subject to the following conditions:     
+  Permission is hereby granted, free of charge, to use and distribute  
+  this software and its documentation without restriction, including   
+  without limitation the rights to use, copy, modify, merge, publish,  
+  distribute, sublicense, and/or sell copies of this work, and to      
+  permit persons to whom this work is furnished to do so, subject to   
+  the following conditions:                                            
    1. The code must retain the above copyright notice, this list of    
       conditions and the following disclaimer.                         
    2. Any modifications must be clearly marked as such.                
    3. Original authors' names are not deleted.                         
-  This software may not be used for commercial purposes without        
-  specific prior written permission from the authors.                  
+   4. The authors' names are not used to endorse or promote products   
+      derived from this software without specific prior written        
+      permission.                                                      
                                   
   THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        
   DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      
@@ -404,7 +576,7 @@ Play given wavefile"
   corresponding to the word related to this syllable.")
 
 ;;;
-;;;  I treid some tests on the resulting speed both runtime and loadtime
+;;;  I tried some tests on the resulting speed both runtime and loadtime
 ;;;  but compiled files don't seem to make any significant difference
 ;;;
 (define (compile_library)
@@ -428,4 +600,16 @@ Compile all the scheme files in the library directory."
      "tobi.scm" "f2bf0lr.scm"))
   t)
 
+(define (register_unisyn_features ddd)
+  (format t "********************\n")
+  (format t "\n")
+  (format t "The Scheme `register_unisyn_features' is no longer required\n")
+  (format t "its function is absorbed into the initialization process\n")
+  (format t "delete you call to this function now as this will cause\n")
+  (format t "an error in the future\n")
+  (format t "\n")
+  (system "sleep 3")
+  (format t "********************\n")
+)  
+
 (provide 'festival)
